/*
 * Barbieru Irineu, 332 CB
 * Proiectarea Cu Microprocesoare
 * Memory Game
 */

#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>

#define INITIAL_SEQ_SIZE 2
#define MAX_SEQ_SIZE 8
#define ROUNDS_PER_STAGE 3

int pini_led[10] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PC6, PC7};
int pini_but[10] = {PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7, PD0, PD1};


void led_on(int index) {
	if (index < 8) /* DDRA */
		PORTA |= (1 << pini_led[index]);
	else /* DDRC */
		PORTC |= (1 << pini_led[index]);
	
	_delay_ms(300);
	
	if (index < 8) /* DDRA */
		PORTA &= ~(1 << pini_led[index]);
	else /* DDRC */
		PORTC &= ~(1 << pini_led[index]);

	_delay_ms(300);
}

int getIndexOfButtonPressed() {
	
	int i;
	
	while (1) {
	
		for (i = 0; i < 10; i++) {
			if (i < 8) {
				if ((PINB & (1 << pini_but[i])) == 0) {
					/* daca se tine apasat butonul, sa nu se considere ca s-a apasat de mai multe ori */
					while ((PINB & (1 << pini_but[i])) == 0);
					return i;
				}
			}
			else {
				if ((PIND & (1 << pini_but[i])) == 0) {
					/* daca se tine apasat butonul, sa nu se considere ca s-a apasat de mai multe ori */
					while ((PIND & (1 << pini_but[i])) == 0);
					return i;
				}
			}
		}
	}

}

void initializare() {
	int i;
	
	/* setez pinii corespunzatori led-urilor, ca iesiri */
	for (i = 0; i < 10; i++) {
		if (i < 8)
			DDRA |= (1 << pini_led[i]);
		else
			DDRC |= (1 << pini_led[i]);
	}
	
	/* setez pinii corespunzatori butoanelor, ca intrari */
	for (i = 0; i < 10; i++) {
		if (i < 8)
			DDRB &= ~(1 << pini_but[i]);
		else
			DDRD &= ~(1 << pini_but[i]);
	}
	/* activez rezistentele de pull-up pentru pinii corespunzatori butoanelor */
	for (i = 0; i < 10; i++) {
		if (i < 8)
			PORTB |= (1 << pini_but[i]);
		else
			PORTD |= (1 << pini_but[i]);
	}

}

void start_or_final_seq() {
	int i;
	
	for (i = 0; i < 5; i++) {
		if (i < 8)
			PORTA |= (1 << pini_led[i]);
		else
			PORTC |= (1 << pini_led[i]);
		if (9 - i < 8)
			PORTA |= (1 << pini_led[9 - i]);
		else
			PORTC |= (1 << pini_led[9 - i]);
		_delay_ms(200);
	}

	_delay_ms(800);

	for (i = 0; i < 10; i++) {
		if (i < 8)
			PORTA &= ~(1 << pini_led[i]);
		else
			PORTC &= ~(1 << pini_led[i]);
		if (9 - i < 8)
			PORTA &= ~(1 << pini_led[9 - i]);
		else
			PORTC &= ~(1 << pini_led[9 - i]);
		_delay_ms(200);
	}
}

void castig_runda() {
	int i, j;

	for (j = 0; j < 2; j++) {
		for (i = 0; i < 10; i++) {
			if (i < 8)
				PORTA |= (1 << pini_led[i]);
			else
				PORTC |= (1 << pini_led[i]);

			_delay_ms(100);

			if (i < 8)
				PORTA &= ~(1 << pini_led[i]);
			else
				PORTC &= ~(1 << pini_led[i]);
		}
	}
}

void pierdut_runda() {
	int i, j;
	
	for (j = 0; j < 4; j++) {

		for (i = 0; i < 10; i++) {
			if (i < 8)
				PORTA |= (1 << pini_led[i]);
			else
				PORTC |= (1 << pini_led[i]);
		}

		_delay_ms(100);

		for (i = 0; i < 10; i++) {
			if (i < 8)
				PORTA &= ~(1 << pini_led[i]);
			else
				PORTC &= ~(1 << pini_led[i]);
		}

		_delay_ms(100);
	}
}

void sfarsit_etapa() {
	int i;

	for (i = 0; i < 10; i++) {
		if (i < 8)
			PORTA |= (1 << pini_led[i]);
		else
			PORTC |= (1 << pini_led[i]);
		_delay_ms(200);
	}

	_delay_ms(800);

	for (i = 0; i < 10; i++) {
		if (i < 8)
			PORTA &= ~(1 << pini_led[i]);
		else
			PORTC &= ~(1 << pini_led[i]);
	}
}

void run_game() {
	int i, size = INITIAL_SEQ_SIZE;
	int win_round, per_etapa_cnt = 0;
	
	initializare();
	
	srand(0);
	
	start_or_final_seq();
	
	while (1) {

		int seq[size];
		
		win_round = 1;
		
		for (i = 0; i < size; i++) {
			seq[i] = rand() % 10;
		}
		
		for (i = 0; i < size; i++) {
			led_on(seq[i]);
		}
		
		for (i = 0; i < size; i++) {
			
			if (getIndexOfButtonPressed() != seq[i]) {
				pierdut_runda();
				win_round = 0;
				per_etapa_cnt = 0;
				break;
			}
			else {
				led_on(seq[i]);
			}
		
		}
		if (win_round == 1) {

			per_etapa_cnt++;

			if (per_etapa_cnt == ROUNDS_PER_STAGE) {
				
				if(size == MAX_SEQ_SIZE) {
					start_or_final_seq();
					break;
				}
				size++;
				per_etapa_cnt = 0;
				sfarsit_etapa();
			}
			else {
				castig_runda();
			}
		}
	
		_delay_ms(1000);
	}

}

int main(void) {


	run_game();
	
	return 0;
}

